class GherkinLine:
    def __init__(self, line_text, line_number):
        self._line_text = line_text
        self._line_number = line_number
        self._trimmed_line_text = line_text.lstrip()
        self.indent = len(line_text) - len(self._trimmed_line_text)

    def get_rest_trimmed(self, length):
        return self._trimmed_line_text[length:].strip()

    def get_line_text(self, indent_to_remove=-1):
        if indent_to_remove < 0 or indent_to_remove > self.indent:
            return self._trimmed_line_text
        else:
            return self._line_text[indent_to_remove:]

    def is_empty(self):
        return not self._trimmed_line_text

    def startswith(self, prefix):
        return self._trimmed_line_text.startswith(prefix)

    def startswith_title_keyword(self, keyword):
        return self._trimmed_line_text.startswith(keyword + ':')

    def table_cells(self):
        column = self.indent + 1
        items = self._trimmed_line_text.strip().split('|')
        cells = []
        for item in items[1:-1]:
            cell_indent = len(item) - len(item.lstrip()) + 1
            cells.append({'column': column + cell_indent, 'text': item.strip()})
            column += len(item) + 1
        return cells

    def tags(self):
        column = self.indent + 1
        items = self._trimmed_line_text.strip().split('@')
        tags = []
        for item in items[1:]:
            tags.append({'column': column, 'text': '@' + item.strip()})
            column += len(item) + 1
        return tags



import os
import json


class Dialect:
    DIALECTS = json.load(open(os.path.join(os.path.dirname(__file__),
                                           'gherkin-languages.json'), 'r'), encoding='utf-8')

    @classmethod
    def for_name(cls, name):
        return cls(cls.DIALECTS[name]) if name in cls.DIALECTS else None

    def __init__(self, spec):
        self.spec = spec

    def feature_keywords(self):
        return self.spec['feature']

    def scenario_keywords(self):
        return self.spec['scenario']

    def scenario_outline_keywords(self):
        return self.spec['scenarioOutline']

    def background_keywords(self):
        return self.spec['background']

    def examples_keywords(self):
        return self.spec['examples']

    def given_keywords(self):
        return self.spec['given']

    def when_keywords(self):
        return self.spec['when']

    def then_keywords(self):
        return self.spec['then']

    def and_keywords(self):
        return self.spec['and']

    def but_keywords(self):
        return self.spec['but']



# coding: utf-8
from distutils.core import setup
setup(
  name = 'gherkin3',
  packages = ['gherkin3'],
  version = '3.0.0',
  description = 'Gherkin parser',
  author = 'Björn Rasmusson',
  author_email = 'cukes@googlegroups.com',
  url = 'https://github.com/cucumber/gherkin-python',
  license = 'MIT',
  download_url = 'https://github.com/cucumber/gherkin-ruby/archive/v3.0.0.tar.gz',
  keywords = ['gherkin', 'cucumber', 'bdd'],
  classifiers = [],
)



import os
from .token import Token
from .gherkin_line import GherkinLine
try:
    python2 = True
    from cStringIO import StringIO
except ImportError:
    python2 = False
    from io import StringIO


class TokenScanner:
    def __init__(self, io):
        if isinstance(io, str):
            if os.path.exists(io):
                self.io = open(io, 'rU')
            else:
                self.io = StringIO(io)
        self.line_number = 0

    def read(self):
        self.line_number += 1
        location = {'line': self.line_number}
        line = self.io.readline()
        if python2:
            line = line.decode('utf-8')
        return Token((GherkinLine(line, self.line_number) if line else line), location)
